==> kube-integration-1.0.11.tgz: package/server/api/kube-native/secrets/controller.js <==
'use strict';

const _                          = require('lodash');
const Promise                    = require('bluebird');
const logger                     = require('cf-logs');
const RegistryCredentialsManager = require('registries-credentials');
const ResponseHandler            = require('./../../../helpers/ResponseHandler');
const BaseCrudController         = require('./../../crud.base.controller');
const globalLabels               = require('./../../../../lib/labels');
const secretLabels               = require('./../../../../lib/kube-native/secret/secret.labels');

class Controller extends BaseCrudController {
    constructor() {
        super({
            lookForNamespaces: true
        });
    }

    /**
     *
     * @param request
     * @param response
     * @param next
     * @flow:
     *  Get the registry name from the body
     *  Ensure the registry exist on accounts registries
     *  Check if secret exist on the cluster
     *  Get credentials to the registry
     *  Build secret
     */
    createImagePullSecret(request, response, next) {
        const account            = request.account;
        const user               = request.user;
        const accountsRegistries = _.get(request, 'body.accountRegistries', []);
        const registryName       = _.get(request, 'body.registry');
        const namespaces         = request.namespaces;
        const kubeManager        = request.kubeManager;
        const secretController   = kubeManager.getSecretController.bind(kubeManager);


        Promise.map(accountsRegistries, (registry) => {
            if (registry.name === registryName) {
                return registry;
            }
        })
            .then(_.compact)
            .then(_.head) // should be only one
            .then((registry) => {
                if (!registry) {
                    throw new Error(`${registryName} not found for account ${account._id}`);
                }
                return registry;
            })
            .then((registry) => {
                const manager = RegistryCredentialsManager.create(registry);
                return Promise.all([
                    manager.getDomain(),
                    manager.getCredentials(),
                    Promise.resolve(registry)
                ]);
            })
            .then(([domain, credentials, usersRegistry]) => {
                return Promise.map((namespaces), (ns) => {

                    const candidateName = secretController(ns).buildImagePullSecretName({
                        domain,
                        ns,
                        name: registryName
                    });
                    const opt           = {
                        domain,
                        credentials,
                        usersRegistry,
                        name: candidateName,
                        ns,
                        labels: {
                            [globalLabels.ACCOUNT_NAME.key]: account.name,
                            [globalLabels.USER_NAME.key]: user.userName,
                        }
                    };

                    return secretController(ns)
                        .describe(candidateName)
                        .catch((err) => {
                            //check error for not found
                            if (err.code === 404) {
                                return;
                            }
                            throw err;
                        })
                        .then((secret) => {
                            if (!secret) {
                                return secretController(ns).createImagePullSecret(opt);
                            } else {
                                const expectedHash = secretController(ns).buildImagePullSecretHash({
                                    username: credentials.username,
                                    password: credentials.password
                                });
                                const currentHash  = secret.getLabelByKey(secretLabels.IMAGE_PULL_SECRET_HASH.key);
                                if (currentHash === expectedHash) {
                                    return Promise.resolve(secret);
                                } else {
                                    return secretController(ns).delete(opt)
                                        .catch((err) => {
                                            // some error occured, continure anyway on creation process
                                            // TODO: send err to monitor
                                            logger.error(err.stack);
                                            return;
                                        })
                                        .then(() => {
                                            return secretController(ns).createImagePullSecret(opt);
                                        });
                                }
                            }
                        });
                });

            })
            .map((secret) => secret.getBasicData())
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data);
                }
                return data;
            })
            .then(ResponseHandler.respondWithResult(response, 201))
            .done(next, next);
    }

}

module.exports = new Controller();

==> kube-integration-1.0.12.tgz: package/server/api/kube-native/secrets/controller.js <==
'use strict';

const _                          = require('lodash');
const Promise                    = require('bluebird');
const logger                     = require('cf-logs');
const RegistryCredentialsManager = require('registries-credentials');
const ResponseHandler            = require('./../../../helpers/ResponseHandler');
const BaseCrudController         = require('./../../crud.base.controller');
const globalLabels               = require('./../../../../lib/labels');
const secretLabels               = require('./../../../../lib/kube-native/secret/secret.labels');

class Controller extends BaseCrudController {
    constructor() {
        super({
            lookForNamespaces: true
        });
    }

    /**
     *
     * @param request
     * @param response
     * @param next
     * @flow:
     *  Get the registry name from the body
     *  Ensure the registry exist on accounts registries
     *  Check if secret exist on the cluster
     *  Get credentials to the registry
     *  Build secret
     */
    createImagePullSecret(request, response, next) {
        const account            = request.account;
        const user               = request.user;
        const bodyRegistries     = _.get(request, 'body.accountRegistries');
        const resitries          = _.get(account, 'integrations.registries', []);
        const accountsRegistries = bodyRegistries ? bodyRegistries : resitries;
        const registryName       = _.get(request, 'body.registry');
        const namespaces         = request.namespaces;
        const kubeManager        = request.kubeManager;
        const secretController   = kubeManager.getSecretController.bind(kubeManager);


        Promise.map(accountsRegistries, (registry) => {
            if (registry.name === registryName) {
                return registry;
            }
        })
            .then(_.compact)
            .then(_.head) // should be only one
            .then((registry) => {
                if (!registry) {
                    throw new Error(`${registryName} not found for account ${account._id}`);
                }
                return registry;
            })
            .then((registry) => {
                const manager = RegistryCredentialsManager.create(registry);
                return Promise.all([
                    manager.getDomain(),
                    manager.getCredentials(),
                    Promise.resolve(registry)
                ]);
            })
            .then(([domain, credentials, usersRegistry]) => {
                return Promise.map((namespaces), (ns) => {

                    const candidateName = secretController(ns).buildImagePullSecretName({
                        domain,
                        ns,
                        name: registryName
                    });
                    const opt           = {
                        domain,
                        credentials,
                        usersRegistry,
                        name: candidateName,
                        ns,
                        labels: {
                            [globalLabels.ACCOUNT_NAME.key]: account.name,
                            [globalLabels.USER_NAME.key]: user.userName,
                        }
                    };

                    return secretController(ns)
                        .describe(candidateName)
                        .catch((err) => {
                            //check error for not found
                            if (err.code === 404) {
                                return;
                            }
                            throw err;
                        })
                        .then((secret) => {
                            if (!secret) {
                                return secretController(ns).createImagePullSecret(opt);
                            } else {
                                const expectedHash = secretController(ns).buildImagePullSecretHash({
                                    username: credentials.username,
                                    password: credentials.password
                                });
                                const currentHash  = secret.getLabelByKey(secretLabels.IMAGE_PULL_SECRET_HASH.key);
                                if (currentHash === expectedHash) {
                                    return Promise.resolve(secret);
                                } else {
                                    return secretController(ns).delete(candidateName)
                                        .catch((err) => {
                                            // some error occured, continure anyway on creation process
                                            // TODO: send err to monitor
                                            logger.error(err.stack);
                                            return;
                                        })
                                        .then(() => {
                                            return secretController(ns).createImagePullSecret(opt);
                                        });
                                }
                            }
                        });
                });

            })
            .map((secret) => secret.getBasicData())
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data);
                }
                return data;
            })
            .then(ResponseHandler.respondWithResult(response, 201))
            .done(next, next);
    }

}

module.exports = new Controller();
